@@ -49,12 +49,16 @@ const char  *gnome_vfs_get_mime_type_from_uri			(GnomeVFSURI       *uri);
 const char  *gnome_vfs_get_mime_type_from_file_data		(GnomeVFSURI       *uri);
 #endif
 
-const char  *gnome_vfs_get_file_mime_type 			(const char        *path, 
-								 const struct stat *optional_stat_info,
-								 gboolean           suffix_only);
+const char * gnome_vfs_get_file_mime_type_fast      (const char        *path,
+						     const struct stat *optional_stat_info);
+const char  *gnome_vfs_get_file_mime_type           (const char        *path,
+						     const struct stat *optional_stat_info,
+						     gboolean           suffix_only);
+gboolean     gnome_vfs_mime_type_is_supertype       (const char        *mime_type);
+char	    *gnome_vfs_get_supertype_from_mime_type (const char        *mime_type);
 
-gboolean    gnome_vfs_mime_type_is_supertype			(const char        *mime_type);
-char	    *gnome_vfs_get_supertype_from_mime_type		(const char        *mime_type);							 
+void         gnome_vfs_mime_info_cache_reload (const char *dir);
+void         gnome_vfs_mime_reload            (void);
 
 G_END_DECLS
 
