@@ -93,7 +93,7 @@ typedef struct {
 	char bv[32];
 } UriStrspnSet; 
 
-UriStrspnSet uri_strspn_sets[] = {
+static UriStrspnSet uri_strspn_sets[] = {
 	{":@]" GNOME_VFS_URI_PATH_STR, FALSE, ""},
 	{"@" GNOME_VFS_URI_PATH_STR, FALSE, ""},
 	{":" GNOME_VFS_URI_PATH_STR, FALSE, ""},
@@ -163,7 +163,7 @@ split_toplevel_uri (const gchar *path, guint path_len,
 	success = FALSE;
 
 	if (path == NULL || path_len == 0) {
-		return NULL;
+		return g_strdup ("/");
 	}
 	
 
@@ -358,7 +358,7 @@ set_uri_element (GnomeVFSURI *uri,
 	char *escaped_text;
 
 	if (text == NULL || len == 0) {
-		uri->text = g_strdup("/");;
+		uri->text = g_strdup ("/");
 		return;
 	}
 
@@ -796,7 +796,7 @@ make_full_uri_from_relative (const char *base_uri, const char *uri)
 			separator = strrchr (mutable_base_uri, '/');
 			if (separator) {
 				/* Make sure we don't eat a domain part */
-				gchar *tmp = (gchar*)((int)separator - 1);
+				char *tmp = separator - 1;
 				if ((separator != mutable_base_uri) && (*tmp != '/')) {
 					*separator = '\0';
 				}
