@@ -60,6 +60,9 @@ struct _GnomeVFSVolumePrivate {
 	char *device_path;
 	dev_t unix_device;
 
+	/* Only for HAL devices: */
+	char *hal_udi;
+
 	/* Only for connected servers */
 	char *gconf_id;
 };
@@ -68,7 +71,7 @@ struct _GnomeVFSVolumePrivate {
 struct _GnomeVFSDrivePrivate {
 	gulong id;
 	GnomeVFSDeviceType device_type;
-	GnomeVFSVolume *volume; /* Owning ref */
+	GList *volumes; /* GnomeVFSVolume list (Owning ref) */
 
 	/* Only for unix mounts: */
 	char *device_path;
@@ -79,13 +82,18 @@ struct _GnomeVFSDrivePrivate {
 	
 	gboolean is_user_visible;
 	gboolean is_connected;
+
+	/* Only for HAL devices: */
+	char *hal_udi;
+
+	gboolean must_eject_at_unmount;
 };
 
 void _gnome_vfs_volume_set_drive                (GnomeVFSVolume        *volume,
 						 GnomeVFSDrive         *drive);
-void _gnome_vfs_drive_set_mounted_volume        (GnomeVFSDrive         *drive,
+void _gnome_vfs_drive_add_mounted_volume        (GnomeVFSDrive         *drive,
 						 GnomeVFSVolume        *volume);
-void _gnome_vfs_drive_unset_volume              (GnomeVFSDrive         *drive,
+void _gnome_vfs_drive_remove_volume             (GnomeVFSDrive         *drive,
 						 GnomeVFSVolume        *volume);
 void _gnome_vfs_volume_unset_drive              (GnomeVFSVolume        *volume,
 						 GnomeVFSDrive         *drive);
@@ -124,6 +132,20 @@ GnomeVFSDrive * _gnome_vfs_volume_monitor_find_fstab_drive_by_activation_uri (Gn
 GnomeVFSVolume *_gnome_vfs_volume_monitor_find_connected_server_by_gconf_id  (GnomeVFSVolumeMonitor *volume_monitor,
 									      const char            *id);
 
+#ifdef USE_HAL
+GnomeVFSVolume *_gnome_vfs_volume_monitor_find_volume_by_hal_udi (GnomeVFSVolumeMonitor *volume_monitor,
+								  const char            *hal_udi);
+GnomeVFSDrive *_gnome_vfs_volume_monitor_find_drive_by_hal_udi   (GnomeVFSVolumeMonitor *volume_monitor,
+								  const char            *hal_udi);
+
+#endif /* USE_HAL */
+
+GnomeVFSVolume *_gnome_vfs_volume_monitor_find_volume_by_device_path (GnomeVFSVolumeMonitor *volume_monitor,
+								      const char            *device_path);
+GnomeVFSDrive *_gnome_vfs_volume_monitor_find_drive_by_device_path   (GnomeVFSVolumeMonitor *volume_monitor,
+								      const char            *device_path);
+
+
 
 char *_gnome_vfs_volume_monitor_uniquify_volume_name (GnomeVFSVolumeMonitor *volume_monitor,
 						      const char            *name);
