@@ -64,6 +64,12 @@ enum {
 	DEFAULT_SIZE_OVERHEAD = 1024
 };
 
+/* When the total copy size is over this limit, we try to
+ * forget the data cached for the files, in order to
+ * lower VM pressure
+ */
+#define DROP_CACHE_SIZE_LIMIT (20*1024*1024)
+
 /* in asynch mode the progress callback does a context switch every time
  * it gets called. We'll only call it every now and then to not loose a
  * lot of performance
@@ -1201,6 +1207,7 @@ copy_file_data (GnomeVFSHandle *target_handle,
 	gpointer buffer;
 	const char *write_buffer;
 	GnomeVFSFileSize total_bytes_read;
+	gboolean forget_cache;
 
 	*skip = FALSE;
 
@@ -1210,6 +1217,9 @@ copy_file_data (GnomeVFSHandle *target_handle,
 
 	buffer = g_malloc (block_size);
 	total_bytes_read = 0;
+
+	/* Enable streaming if the total size is large */
+	forget_cache = progress->progress_info->bytes_total >= DROP_CACHE_SIZE_LIMIT;
 	
 	do {
 		GnomeVFSFileSize bytes_read;
@@ -1227,6 +1237,10 @@ copy_file_data (GnomeVFSHandle *target_handle,
 
 			result = gnome_vfs_read (source_handle, buffer,
 						 block_size, &bytes_read);
+			if (forget_cache) {
+				gnome_vfs_forget_cache (source_handle,
+							total_bytes_read, bytes_read);
+			}
 			if (result != GNOME_VFS_OK && result != GNOME_VFS_ERROR_EOF)
 				retry = handle_error (&result, progress,
 						      error_mode, skip);
@@ -1236,7 +1250,7 @@ copy_file_data (GnomeVFSHandle *target_handle,
 			break;
 		}
 
-		total_bytes_read += total_bytes_read;
+		total_bytes_read += bytes_read;
 		bytes_to_write = bytes_read;
 
 		progress->progress_info->phase = GNOME_VFS_XFER_PHASE_WRITETARGET;
@@ -1256,6 +1270,11 @@ copy_file_data (GnomeVFSHandle *target_handle,
 			bytes_to_write -= bytes_written;
 			write_buffer += bytes_written;
 		} while ((result == GNOME_VFS_OK && bytes_to_write > 0) || retry);
+		
+		if (forget_cache && bytes_to_write == 0) {
+			gnome_vfs_forget_cache (target_handle,
+						total_bytes_read - bytes_read, bytes_read);
+		}
 
 		progress->progress_info->phase = GNOME_VFS_XFER_PHASE_COPYING;
 
@@ -1301,7 +1320,7 @@ xfer_open_source (GnomeVFSHandle **source_handle,
 {
 	GnomeVFSResult result;
 	gboolean retry;
-
+	
 	*skip = FALSE;
 	do {
 		retry = FALSE;
@@ -1527,9 +1546,11 @@ copy_directory (GnomeVFSFileInfo *source_file_info,
 	GnomeVFSResult result;
 	GnomeVFSDirectoryHandle *source_directory_handle;
 	GnomeVFSDirectoryHandle *dest_directory_handle;
+	GnomeVFSFileInfo *target_dir_info;
 
 	source_directory_handle = NULL;
 	dest_directory_handle = NULL;
+	target_dir_info = NULL;
 	
 	result = gnome_vfs_directory_open_from_uri (&source_directory_handle, source_dir_uri, 
 						    GNOME_VFS_FILE_INFO_DEFAULT);
@@ -1563,6 +1584,14 @@ copy_directory (GnomeVFSFileInfo *source_file_info,
 		return result;
 	}
 
+	target_dir_info = gnome_vfs_file_info_new ();
+	result = gnome_vfs_get_file_info_uri (target_dir_uri, target_dir_info,
+                                        GNOME_VFS_FILE_INFO_DEFAULT);
+	if (result != GNOME_VFS_OK) {
+		gnome_vfs_file_info_unref (target_dir_info);
+		target_dir_info = NULL;
+	}
+
 	if (call_progress_with_uris_often (progress, source_dir_uri, target_dir_uri, 
 					   GNOME_VFS_XFER_PHASE_OPENTARGET) != 0) {
 
@@ -1589,6 +1618,9 @@ copy_directory (GnomeVFSFileInfo *source_file_info,
 				gnome_vfs_file_info_unref (info);	
 				break;
 			}
+			if (target_dir_info && GNOME_VFS_FILE_INFO_SGID(target_dir_info)) {
+				info->gid = target_dir_info->gid;
+			}
 			
 			/* Skip "." and "..".  */
 			if (strcmp (info->name, ".") != 0 && strcmp (info->name, "..") != 0) {
@@ -1650,20 +1682,29 @@ copy_directory (GnomeVFSFileInfo *source_file_info,
 	gnome_vfs_directory_close (source_directory_handle);
 
 	if (result == GNOME_VFS_OK) {
+		GnomeVFSFileInfo *info;
+		info = gnome_vfs_file_info_new ();
 		/* FIXME the modules should ignore setting of permissions if
 		 * "valid_fields & GNOME_VFS_FILE_INFO_FIELDS_PERMISSIONS" is clear
 		 * for now, make sure permissions aren't set to 000
 		 */
-
+		gnome_vfs_file_info_copy (source_file_info, info);
+		if (target_dir_info && GNOME_VFS_FILE_INFO_SGID(target_dir_info)) {
+			info->gid = target_dir_info->gid;
+		}
 		if ((source_file_info->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_PERMISSIONS) != 0) {
 			/* Call this separately from the time one, since one of them may fail,
 			   making the other not run. */
-			gnome_vfs_set_file_info_uri (target_dir_uri, source_file_info, 
+			gnome_vfs_set_file_info_uri (target_dir_uri, info, 
 						     GNOME_VFS_SET_FILE_INFO_OWNER | GNOME_VFS_SET_FILE_INFO_PERMISSIONS);
 		}
 		
 		/* Call this last so nothing else changes the times */
-		gnome_vfs_set_file_info_uri (target_dir_uri, source_file_info, GNOME_VFS_SET_FILE_INFO_TIME);
+		gnome_vfs_set_file_info_uri (target_dir_uri, info, GNOME_VFS_SET_FILE_INFO_TIME);
+		gnome_vfs_file_info_unref (info);
+	}
+	if (target_dir_info) {
+		gnome_vfs_file_info_unref (target_dir_info);
 	}
 
 	return result;
@@ -1713,6 +1754,17 @@ copy_items (const GList *source_uri_list,
 			((GnomeVFSURI *)target_item->data);
 
 		if (result == GNOME_VFS_OK) {
+			GnomeVFSFileInfo *target_dir_info;
+			/* get target_dir URI and file info to take care of SGID */
+			target_dir_info = gnome_vfs_file_info_new ();
+			result = gnome_vfs_get_file_info_uri (target_dir_uri, target_dir_info,
+																						GNOME_VFS_FILE_INFO_DEFAULT);
+			if (result == GNOME_VFS_OK && GNOME_VFS_FILE_INFO_SGID(target_dir_info)) {
+				info->gid = target_dir_info->gid;
+			}
+			gnome_vfs_file_info_unref (target_dir_info);
+			result = GNOME_VFS_OK;
+
 			/* optionally keep trying until we hit a unique target name */
 			for (count = 1; ; count++) {
 				GnomeVFSXferOverwriteMode overwrite_mode_abort;
