@@ -419,7 +419,7 @@ static void construct_gl_http_proxy(gboolean use_proxy)
 	    gconf_client_get_int(gl_client,
 				 KEY_GCONF_HTTP_PROXY_PORT, NULL);
 
-	if (proxy_host) {
+	if (proxy_host && proxy_host[0] != '\0') {
 	    if (0 != proxy_port && 0xffff >= (unsigned) proxy_port) {
 		gl_http_proxy =
 		    g_strdup_printf("%s:%u", proxy_host, (unsigned)
@@ -524,7 +524,7 @@ notify_gconf_value_changed(GConfClient * client,
     }
 }
 
-void proxy_init()
+void proxy_init(void)
 {
     GError *gconf_error = NULL;
     gboolean use_proxy;
@@ -581,7 +581,7 @@ void proxy_init()
 }
 
 
-void proxy_shutdown()
+void proxy_shutdown(void)
 {
     g_object_unref(G_OBJECT(gl_client));
 }
