@@ -20,6 +20,7 @@
    Author: Bastien Nocera <hadess@hadess.net>
 */
 
+#include <locale.h>
 #include <libgnomevfs/gnome-vfs.h>
 
 char *directory;
@@ -110,12 +111,19 @@ list (void)
 int
 main (int argc, char *argv[])
 {
+  
+  	setlocale (LC_ALL, "");
+
 	gnome_vfs_init ();
 
 	if (argc > 1) {
 		directory = argv[1];
 	} else {
-		directory = g_get_current_dir ();
+		char *tmp;
+
+		tmp = g_get_current_dir ();
+		directory = gnome_vfs_escape_path_string (tmp);
+		g_free (tmp);
 	}
 
 	list ();
